/*  bc635.c - Datum BC635/7 PCI VxWorks Driver */

/*
  *******************************************************************************
** Copyright (C) 2000 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/


#ifndef __INCBC635c
#define __INCBC635c

#ifdef __cplusplus
extern  "C"
{
#endif

/*
modification history
--------------------
12jun00,spk: index arg in tfpDevInstall now used to build device name
             (ex, index = 1 ===> devName = "/tfp1")
*/

/*
Description

This file contains

*/

#include "vxWorks.h"
#include "stdio.h"
#include "logLib.h"
#include "sysLib.h"
#include "drv/pci/pciIomapLib.h"
#include "taskLib.h"
#include "string.h"
#include "bc635P.h"
#include "intLib.h"
#include "iv.h"

/* external declarations */

/* local declarations */

/* imports */

    IMPORT int bc635Debug;
    IMPORT int bc635IntsDebug;
    IMPORT int bc635PciMemOffset;

/* global declarations */
    int     TfpDrvNum;

/* forward declarations */
    VOID    tfpDrvIsr
            (
                     TFP_DEV * pDev
    );
    LOCAL VOID isrPoll (TFP_DEV * pDev);



/*******************************************************************************
*
* tfpDrv - install bc635/7(TFP) vxWorks driver
*
* This routine ....
* next line.
*
* RETURNS: OK, or ERROR if
* next line.
*
*/

    int
            tfpDrv (void)
    {

        TfpDrvNum = iosDrvInstall ((FUNCPTR) NULL,
                                   (FUNCPTR) NULL,
                                   (FUNCPTR) tfpOpen,
                                   (FUNCPTR) tfpClose,
                                   (FUNCPTR) NULL,
                                   (FUNCPTR) NULL,
                                   (FUNCPTR) tfpIoctl);

        return (TfpDrvNum);

    }




/*******************************************************************************
*
* tfpDrvInstall - install bc635/7(TFP)_ vxWorks driver support
*
* This routine ....
* next line.
*
* RETURNS: OK, or ERROR if
* next line.
*
*/

            STATUS
            tfpDevInstall (int index, int intvec)
    {
        int     pciBusNo;
        int     pciDevNo;
        int     pciFuncNo;
        TFP_DEV *pTfpDev;
        UINT32  addr;
        char    model[80];
        char    devName[80];


        /* get bus/device/function numbers */
        if      (pciFindDevice (BC635_PCI_VENDOR_ID, BC635_PCI_DEVICE_ID, index,
                                 &pciBusNo, &pciDevNo, &pciFuncNo) == ERROR)
        {
            TFP_MSG ("BC635/7(TFP): device not installed\n", 0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

        /* device exists - allocate vxWorks driver dev header */
        if      ((pTfpDev = malloc (sizeof (TFP_DEV))) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): malloc0 failed\n", 0, 0, 0, 0, 0, 0);
            return (ERROR);
        }


        /* Clear some fields */
        pTfpDev->datamutex = NULL;
        pTfpDev->eventmutex = NULL;
        pTfpDev->timemutex = NULL;
        pTfpDev->cmdmutex = NULL;
        pTfpDev->eventSem = NULL;
        pTfpDev->periodicSem = NULL;
        pTfpDev->onePpsSem = NULL;
        pTfpDev->gpsSem = NULL;
        pTfpDev->strobeSem = NULL;
        pTfpDev->inthook = (VOIDFUNCPTR) NULL;
        pTfpDev->expectedInts = 0;



        /* get register base addr (BAR0) */
        pciConfigInLong (pciBusNo, pciDevNo, pciFuncNo,
                         PCI_CFG_BASE_ADDRESS_0, &addr);

        /* convert pci bus based address to address as seen from cpu */
        pTfpDev->baseAdrsRegs = bc635PciMemOffset + addr;

        TFP_DEBUG_MSG ("tfp: regs base addrs (as seen from cpu): 0x%08X\n",
                       pTfpDev->baseAdrsRegs, 0, 0, 0, 0, 0);


        /* get dpram base addr (BAR1) */
        pciConfigInLong (pciBusNo, pciDevNo, pciFuncNo,
                         PCI_CFG_BASE_ADDRESS_1, &addr);

        /* convert pci bus based address to address as seen from cpu */
        pTfpDev->baseAdrsDpram = bc635PciMemOffset + addr;

        TFP_DEBUG_MSG ("tfp: dpram base addrs (as seen from cpu): 0x%08X\n",
                       pTfpDev->baseAdrsDpram, 0, 0, 0, 0, 0);


        /* determine address of tfp command 'input' area */
        pTfpDev->baseAdrsInput = pTfpDev->baseAdrsDpram +
            (TFP_IN_BYTE (TFP_DPR_IN_ADRS_MSB (pTfpDev->baseAdrsDpram)) << 8) +
            TFP_IN_BYTE (TFP_DPR_IN_ADRS_LSB (pTfpDev->baseAdrsDpram));

        TFP_DEBUG_MSG ("tfp: dpram input base addrs: 0x%08X\n",
                       pTfpDev->baseAdrsInput, 0, 0, 0, 0, 0);

        /* determine address of tfp command 'output' area */
        pTfpDev->baseAdrsOutput = pTfpDev->baseAdrsDpram +
            (TFP_IN_BYTE (TFP_DPR_OUT_ADRS_MSB (pTfpDev->baseAdrsDpram)) << 8) +
            TFP_IN_BYTE (TFP_DPR_OUT_ADRS_LSB (pTfpDev->baseAdrsDpram));

        TFP_DEBUG_MSG ("tfp: dpram output base addrs: 0x%08X\n",
                       pTfpDev->baseAdrsOutput, 0, 0, 0, 0, 0);

        /* determine address of tfp 'gps packet' area */
        pTfpDev->baseAdrsGpsPkt = pTfpDev->baseAdrsDpram +
            (TFP_IN_BYTE (TFP_DPR_GPS_ADRS_MSB (pTfpDev->baseAdrsDpram)) << 8) +
            TFP_IN_BYTE (TFP_DPR_GPS_ADRS_LSB (pTfpDev->baseAdrsDpram));

        TFP_DEBUG_MSG ("tfp: dpram gps packet base addrs: 0x%08X\n",
                       pTfpDev->baseAdrsGpsPkt, 0, 0, 0, 0, 0);

        /* determine address of tfp 'year' area */
        pTfpDev->baseAdrsYr = pTfpDev->baseAdrsDpram +
            (TFP_IN_BYTE (TFP_DPR_YR_ADRS_MSB (pTfpDev->baseAdrsDpram)) << 8) +
            TFP_IN_BYTE (TFP_DPR_YR_ADRS_LSB (pTfpDev->baseAdrsDpram));

        TFP_DEBUG_MSG ("tfp: dpram year base addrs: 0x%08X\n",
                       pTfpDev->baseAdrsYr, 0, 0, 0, 0, 0);

        /*
         * Verify functionality of tfp command interface by executing 'get
         * model' cmd.
         */
        if (tfpCmdGetTfpModel (pTfpDev, model) == ERROR)
        {
            TFP_MSG ("BC635/7(TFP): command interface failed\n", 0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }

        /*
         * Model string should be: "BC635PCI" or "BC637PCU". Check first 4
         * letters since these are always the same.
         */
        pTfpDev->gps_supported = FALSE;
        if (strncmp ("BC635", model, 5) != 0)
            if (strncmp ("BC637", model, 5) == 0)
            {
                pTfpDev->gps_supported = TRUE;
                TFP_MSG ("BC635/7(TFP): BC637 Detected - GPS Facilities Enabled\n",
                         0, 0, 0, 0, 0, 0);
            }
            else
            {
                TFP_MSG ("BC635/7(TFP): command test failed\n", 0, 0, 0, 0, 0, 0);
                tfpDrvFree (pTfpDev);
                return (ERROR);
            }

        /* Create Various Semaphores needed by Driver */

        if ((pTfpDev->datamutex = semMCreate (SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }

        if ((pTfpDev->eventmutex = semMCreate (SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }
        if ((pTfpDev->timemutex = semMCreate (SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }
        if ((pTfpDev->cmdmutex = semMCreate (SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }


        if ((pTfpDev->eventSem = semBCreate (SEM_EMPTY, SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }

        if ((pTfpDev->periodicSem = semBCreate (SEM_EMPTY, SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }

        if ((pTfpDev->gpsSem = semBCreate (SEM_EMPTY, SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }

        if ((pTfpDev->onePpsSem = semBCreate (SEM_EMPTY, SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }
        if ((pTfpDev->strobeSem = semBCreate (SEM_EMPTY, SEM_Q_FIFO)) == NULL)
        {
            TFP_MSG ("BC635/7(TFP): Can't Allocate Semaphores\n",
                     0, 0, 0, 0, 0, 0);
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }




        (void) sprintf (devName, "/tfp%d", index);
        if (iosDevAdd ((DEV_HDR *) pTfpDev, devName, TfpDrvNum) == ERROR)
        {
            tfpDrvFree (pTfpDev);
            return (ERROR);
        }

        if (intvec == 0)        /* no interrupt vector, use polling */
            taskSpawn ("BC635POLL", 100, 0, 0x1000, (FUNCPTR) isrPoll, (int) pTfpDev, 0, 0, 0, 0, 0, 0, 0, 0, 0);
        else
        {
            intConnect ((VOIDFUNCPTR *) INUM_TO_IVEC (intvec),
                        (VOIDFUNCPTR) tfpDrvIsr, (int) pTfpDev);
            intEnable (intvec);
        }

        return (OK);

    }


    void    tfpDrvFree
            (
                     TFP_DEV * pTfpDev
    )
    {
        if (pTfpDev->datamutex != NULL)
            semDelete (pTfpDev->datamutex);

        if (pTfpDev->eventmutex != NULL)
            semDelete (pTfpDev->eventmutex);

        if (pTfpDev->timemutex != NULL)
            semDelete (pTfpDev->timemutex);

        if (pTfpDev->cmdmutex != NULL)
            semDelete (pTfpDev->cmdmutex);

        if (pTfpDev->eventSem != NULL)
            semDelete (pTfpDev->eventSem);

        if (pTfpDev->periodicSem != NULL)
            semDelete (pTfpDev->periodicSem);

        if (pTfpDev->onePpsSem != NULL)
            semDelete (pTfpDev->onePpsSem);

        if (pTfpDev->gpsSem != NULL)
            semDelete (pTfpDev->gpsSem);

        if (pTfpDev->strobeSem != NULL)
            semDelete (pTfpDev->strobeSem);


        (void) free (pTfpDev);
    }

    LOCAL VOID isrPoll (TFP_DEV * pDev)
    {
        while (0 == 0)
        {
            tfpDrvIsr (pDev);
            taskDelay (1);
        }
    }

/*******************************************************************************
* tfpDrvIsr - driver Interrupt Service Routine
*
*/
    VOID    tfpDrvIsr
            (
                     TFP_DEV * pDev
    )
    {
        UINT32  intstat;
        UINT32  intmask;

        /* make sure we got some interrupt */

                TFP_IN_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), &intstat);
                TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);

        if      (intContext () == FALSE)        /* to allow test of isr in
                                                 * task level */
                    semTake (pDev->datamutex, WAIT_FOREVER);

        /* check for interrupts */
        if      ((intstat & INT_BITMASK) == 0)
        {
            if (intContext () == FALSE)
                semGive (pDev->datamutex);
            return;             /* no interrupts to handle */
        }


        /* call user inthook if present */

        if (pDev->inthook != 0)
        {

            /* call user routine */

            /* get instat again - he may have cleared some bits */
            TFP_IN_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), &intstat);
            TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);

            /* check for interrupts */
            if ((intstat & INT_BITMASK) == 0)
            {
                if (intContext () == FALSE)
                    semGive (pDev->datamutex);
                return;         /* no interrupts to handle */
            }

        }

        intstat = intstat & INT_BITMASK;
        /* process anything he didn't */

        if ((pDev->expectedInts & INT_BITMASK) == 0)
        {

            /*
             * we weren't expecting any interrupts, just clear and return.
             */

            TFP_OUT_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), intstat);
            if (intContext () == FALSE)
                semGive (pDev->datamutex);
            return;
        }


        if (intstat & BCINT_EVENT)
        {
            /* Process EVENT Interrupt */
            if (pDev->expectedInts & BCINT_EVENT)
            {
                pDev->expectedInts = pDev->expectedInts & ~BCINT_EVENT;
                intmask = intmask & ~BCINT_EVENT; 
                TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
 
                semGive (pDev->eventSem);
            }

        }

        if (intstat & INT_PERIODIC)
        {
            /* Process PERIODIC Interrupt */
            if (pDev->expectedInts & INT_PERIODIC)
            {
                pDev->expectedInts = pDev->expectedInts & ~INT_PERIODIC;
                intmask = intmask & ~INT_PERIODIC; 
                TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
 
                semGive (pDev->periodicSem);
            }

        }

        if (intstat & INT_STROBE)
        {
            /* Process STROBE Interrupt */
            if (pDev->expectedInts & INT_STROBE)
            {
                pDev->expectedInts = pDev->expectedInts & ~INT_STROBE;
                intmask = intmask & ~INT_STROBE; 
                TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                semGive (pDev->strobeSem);
            }

        }

        if (intstat & INT_1PPS)
        {
            /* Process 1PPS Interrupt */
            if (pDev->expectedInts & INT_1PPS)
            {
                pDev->expectedInts = pDev->expectedInts & ~INT_1PPS;
                intmask = intmask & ~INT_1PPS; 
                TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
 
                semGive (pDev->onePpsSem);
            }

        }

        if (intstat & INT_PACKET)
        {
            /* Process GPS Packet Interrupt */
            if (pDev->expectedInts & INT_PACKET)
            {
                pDev->expectedInts = pDev->expectedInts & ~INT_PACKET;
                intmask = intmask & ~INT_PACKET;
                TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                semGive (pDev->gpsSem);
            }

        }

        /* Do clear of intstat */
        TFP_OUT_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), intstat);

        if (intContext () == FALSE)
            semGive (pDev->datamutex);
    }


/*******************************************************************************
* tfpOpen - Open an instance of the driver
*
*/
    TFP_DEV *tfpOpen
            (
                     TFP_DEV * pDev,
                     char *remainder,
                     int mode
    )
    {

        TFP_DEBUG_MSG ("tfpOpen: opened device\n", 0, 0, 0, 0, 0, 0);
        return (pDev);

    }


/*******************************************************************************
* tfpClose - close device
*
*/
    STATUS  tfpClose
            (
                     TFP_DEV * pDev
    )
    {
        TFP_DEBUG_MSG ("tfpClose: closed device\n", 0, 0, 0, 0, 0, 0);
        return (OK);
    }


/****************************************************************************
* tfpIoctl - perform I/O control on TFP device
*
*/
    STATUS  tfpIoctl
            (
                     TFP_DEV * pDev,
                     int ioReqCode,
                     int arg
    )
    {
        UINT32  tmMinor;
        UINT32  tmMajor;
        TFP_TV *pTv;
        TFP_TM *pTm;

                TFP_DEBUG_MSG ("tfpIoctl: ioReqCode=%d, arg=x%08X\n", ioReqCode, arg,
                                       0, 0, 0, 0);

        switch  (ioReqCode)
        {

                case SENDGPSPACKET:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                /* stuff cmd arg bytes in tfp input area */
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SENDGPSPACKET);
                {
                    int     i;
                    TFP_GPS_PACKET *packet =
                    (TFP_GPS_PACKET *) arg;

                            TFP_OUT_BYTE (pDev->baseAdrsInput + 1, packet->
                                                  packet_length);

                            TFP_OUT_BYTE (pDev->baseAdrsInput + 2, packet->
                                                  packet_id);

                    for     (i = 0; i < packet->packet_length - 1; i++)
                                TFP_OUT_BYTE (pDev->baseAdrsInput + 3 + i,
                                                    packet->packet_data[i]);

                    if      (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG (" SENDGPSPACKET: timeout waiting for cmd interface\n",
                                       0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }
                }
                if      (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                break;
            case REQGPSPACKET:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                /* Clear out the GPS sem */
                semTake (pDev->gpsSem, NO_WAIT);

                /* Clear bit 2 of ack register */
                TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_GPS_CLR);
                /* stuff cmd arg bytes in tfp input area */
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_REQGPSPACKET);
                {
                    int     i;

                    TFP_GPS_PACKET *packet =
                    (TFP_GPS_PACKET *) arg;

                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, packet->
                                  packet_id);
                  
                    /* Get ready for GPS Interrupt */
                    semTake (pDev->datamutex, WAIT_FOREVER);
                    pDev->expectedInts = pDev->expectedInts | INT_PACKET;
                    semGive (pDev->datamutex);
                    {           /* flip the mask bit on */
                        UINT32  intmask;
 
                        TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);
                        intmask = intmask | INT_PACKET;
                        TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                    }

                    /* Send the Command */
                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG (" SENDGPSPACKET: timeout waiting for cmd interface\n",
                                       0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }

                    /* Interrupt might have already happened, but that's ok */
                    if (semTake (pDev->gpsSem, sysClkRateGet () * 3) == ERROR)  /* Wait up to 3 sec */
                        packet->packet_length = 0;
                    else
                    {


                        packet->packet_length = TFP_IN_BYTE (pDev->baseAdrsGpsPkt);
                        if (packet->packet_length > 0)
                            packet->packet_id = TFP_IN_BYTE (pDev->baseAdrsGpsPkt + 1);

                        for (i = 0; i < packet->packet_length - 1; i++)
                        {
                            packet->packet_data[i] = TFP_IN_BYTE (pDev->baseAdrsGpsPkt + 2 + i);
                        }
                    }
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;


            case MANREQGPSPACKET:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                /* Clear bit 2 of ack register */
                TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_GPS_CLR);
                /* stuff cmd arg bytes in tfp input area */
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_MANREQGPSPACKET);
                {
                    int     i;
                    TFP_GPS_SENDRECEIVE_PACKET *packet =
                    (TFP_GPS_SENDRECEIVE_PACKET *) arg;

                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, packet->
                                  send.packet_length);

                    TFP_OUT_BYTE (pDev->baseAdrsInput + 2, packet->
                                  send.packet_id);

                    TFP_OUT_BYTE (pDev->baseAdrsInput + 3, packet->
                                  receive.packet_length);

                    TFP_OUT_BYTE (pDev->baseAdrsInput + 4, packet->
                                  receive.packet_id);



                    for (i = 0; i < packet->send.packet_length - 1; i++)
                        TFP_OUT_BYTE (pDev->baseAdrsInput + 5
                                      + i,
                                      packet->send.packet_data[i]);
                    /* Get ready for GPS Interrupt */
                    semTake (pDev->datamutex, WAIT_FOREVER);
                    pDev->expectedInts = pDev->expectedInts | INT_PACKET;
                    semGive (pDev->datamutex);
                    {           /* flip the mask bit on */
                        UINT32  intmask;
 
                        TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);
                        intmask = intmask | INT_PACKET;
                        TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                    }
 

                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG (" SENDGPSPACKET: timeout waiting for cmd interface\n",
                                       0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }

                    /* Interrupt might have already happened, but that's ok */

                    if (semTake (pDev->gpsSem, sysClkRateGet () * 3) == ERROR)
                        /* Wait up to 3 sec */
                        packet->receive.packet_length = 0;
                    else
                    {


                        packet->receive.packet_length = TFP_IN_BYTE (pDev->baseAdrsGpsPkt);
                        if (packet->receive.packet_length > 0)
                            packet->receive.packet_id = TFP_IN_BYTE (pDev->baseAdrsGpsPkt + 1);

                        for (i = 0; i < packet->receive.packet_length - 1; i++)
                        {
                            packet->receive.packet_data[i] = TFP_IN_BYTE (pDev->baseAdrsGpsPkt + 2 + i);
                        }
                    }
                }


                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);

                break;
            case BC635_INTHOOK:
                semTake (pDev->datamutex, WAIT_FOREVER);
                pDev->inthook = (VOIDFUNCPTR) arg;
                semGive (pDev->datamutex);
                break;

            case BC635_INT_WAITONGPS:
                semTake (pDev->datamutex, WAIT_FOREVER);
                pDev->expectedInts = pDev->expectedInts | INT_PACKET;
                semGive (pDev->datamutex);
                {               /* flip the mask bit on */
                    UINT32  intmask;

                    TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);
                    intmask = intmask | INT_PACKET;
                    TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                }
                semTake (pDev->gpsSem, WAIT_FOREVER);
                break;

            case BC635_INT_WAITONEVENT:
                semTake (pDev->datamutex, WAIT_FOREVER);
                pDev->expectedInts = pDev->expectedInts | BCINT_EVENT;
                semGive (pDev->datamutex);
                {               /* flip the mask bit on */
                    UINT32  intmask;

                    TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);
                    intmask = intmask | BCINT_EVENT;
                    TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                }
                semTake (pDev->eventSem, WAIT_FOREVER);
                break;

            case BC635_INT_WAITONSTROBE:
                semTake (pDev->datamutex, WAIT_FOREVER);
                pDev->expectedInts = pDev->expectedInts | INT_STROBE;
                semGive (pDev->datamutex);
                {               /* flip the mask bit on */
                    UINT32  intmask;

                    TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);
                    intmask = intmask | INT_STROBE;
                    TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                }
                semTake (pDev->strobeSem, WAIT_FOREVER);
                break;

            case BC635_INT_WAITON1PPS:
                semTake (pDev->datamutex, WAIT_FOREVER);
                pDev->expectedInts = pDev->expectedInts | INT_1PPS;
                semGive (pDev->datamutex);
                {               /* flip the mask bit on */
                    UINT32  intmask;

                    TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);
                    intmask = intmask | INT_1PPS;
                    TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                }
                semTake (pDev->onePpsSem, WAIT_FOREVER);
                break;

            case BC635_INT_WAITONPERIODIC:
                semTake (pDev->datamutex, WAIT_FOREVER);
                pDev->expectedInts = pDev->expectedInts | INT_PERIODIC;
                semGive (pDev->datamutex);
                {               /* flip the mask bit on */
                    UINT32  intmask;

                    TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), &intmask);
                    intmask = intmask | INT_PERIODIC;
                    TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), intmask);
                }
                semTake (pDev->periodicSem, WAIT_FOREVER);
                break;

            case SELTIMINGMODE:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSetTmMode (pDev, arg) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);

                break;


            case SELTIMEFORMAT:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSetTmFmt (pDev, arg) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }

                if (intContext () == FALSE)
                   semGive(pDev->cmdmutex);

                break;


            case TIMEREQUEST:
                TFP_OUT_LONG (TFP_REG_TIMEREQ (pDev->baseAdrsRegs), 0);
                break;

            case EVENTREQUEST:
                TFP_OUT_LONG (TFP_REG_EVENTREQ (pDev->baseAdrsRegs), 0);
                break;

            /* get 'decimal units time' contained in TIME0/1 major,minor regs */
            case RDTIMETM:

                /*
                 * Time is not latched in this routine - use RDTIMEREQ for
                 * latched time
                 */

                /* get minor time */
                TFP_IN_LONG (TFP_REG_TIME0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time */
                TFP_IN_LONG (TFP_REG_TIME1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDTIME): maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* stuff in user time struct */

                pTm = (TFP_TM *) arg;

                /* get decimal secs (0-59) */
                pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

                /* get decimal mins (0-59) */
                pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

                /* get decimal hours (0-23) */
                pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

                /* get decimal year-day (0-366) */
                pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

                pTm->tm.tm_year = (TFP_IN_BYTE (pDev->baseAdrsYr) << 8) +
                    TFP_IN_BYTE (pDev->baseAdrsYr + 1);

                /* usecs */
                pTm->usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTm->status = TFP_TM0_STATUS (tmMinor);

                break;

            case RDEVENTTM:

                /*
                 * Time is not latched in this routine - use RDEVENTREQ for
                 * latched time
                 */

                /* get minor time */
                TFP_IN_LONG (TFP_REG_EVENT0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time */
                TFP_IN_LONG (TFP_REG_EVENT1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDEVENT): maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* stuff in user time struct */

                pTm = (TFP_TM *) arg;

                /* get decimal secs (0-59) */
                pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

                /* get decimal mins (0-59) */
                pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

                /* get decimal hours (0-23) */
                pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

                /* get decimal year-day (0-366) */
                pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

                pTm->tm.tm_year = (TFP_IN_BYTE (pDev->baseAdrsYr) << 8) +
                    TFP_IN_BYTE (pDev->baseAdrsYr + 1);

                /* usecs */
                pTm->usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTm->status = TFP_TM0_STATUS (tmMinor);

                break;

                /*
                 * get 'unix units' time contained in TIME0/1 major,minor
                 * regs
                 */
            case RDTIMETMREQ:

                if (intContext () == FALSE)
                    semTake (pDev->timemutex, WAIT_FOREVER);

                /* latch current minor/major time */
                TFP_OUT_LONG (TFP_REG_TIMEREQ (pDev->baseAdrsRegs), 0);

                /* get minor time */
                TFP_IN_LONG (TFP_REG_TIME0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time */
                TFP_IN_LONG (TFP_REG_TIME1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDTIMEREQ): maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* stuff in user time struct */

                pTm = (TFP_TM *) arg;

                /* get decimal secs (0-59) */
                pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

                /* get decimal mins (0-59) */
                pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

                /* get decimal hours (0-23) */
                pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

                /* get decimal year-day (0-366) */
                pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

                pTm->tm.tm_year = (TFP_IN_BYTE (pDev->baseAdrsYr) << 8) +
                    TFP_IN_BYTE (pDev->baseAdrsYr + 1);

                /* usecs */
                pTm->usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTm->status = TFP_TM0_STATUS (tmMinor);

                if (intContext () == FALSE)
                    semGive (pDev->timemutex);
                break;

            case RDEVENTTMREQ:

                if (intContext () == FALSE)
                    semTake (pDev->eventmutex, WAIT_FOREVER);
                /* latch current minor/major time (returned val not used) */
                TFP_IN_LONG (TFP_REG_EVENTREQ (pDev->baseAdrsRegs), &tmMinor);

                /* get minor time */
                TFP_IN_LONG (TFP_REG_EVENT0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time */
                TFP_IN_LONG (TFP_REG_EVENT1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDEVENTREQ): maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* stuff in user time struct */

                pTm = (TFP_TM *) arg;

                /* get decimal secs (0-59) */
                pTm->tm.tm_sec = TFP_TM1_DEC_SEC (tmMajor);

                /* get decimal mins (0-59) */
                pTm->tm.tm_min = TFP_TM1_DEC_MIN (tmMajor);

                /* get decimal hours (0-23) */
                pTm->tm.tm_hour = TFP_TM1_DEC_HOUR (tmMajor);

                /* get decimal year-day (0-366) */
                pTm->tm.tm_yday = TFP_TM_DEC_YDAY (tmMinor, tmMajor);

                pTm->tm.tm_year = (TFP_IN_BYTE (pDev->baseAdrsYr) << 8) +
                    TFP_IN_BYTE (pDev->baseAdrsYr + 1);

                /* usecs */
                pTm->usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTm->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTm->status = TFP_TM0_STATUS (tmMinor);
                if (intContext () == FALSE)
                    semGive (pDev->eventmutex);

                break;

            case RDTIMETV:


                /*
                 * Time is not latched in this routine - use RDTIMEREQ for
                 * latched time
                 */


                /* get minor time */
                TFP_IN_LONG (TFP_REG_TIME0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time (unix secs (from 12am 1/1/70) */
                TFP_IN_LONG (TFP_REG_TIME1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDTIMETV):maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* translate to user values */

                pTv = (TFP_TV *) arg;

                /* unix secs (secs from 12am 1/1/70) */
                pTv->tv.tv_sec = tmMajor;

                /* usecs */
                pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTv->status = TFP_TM0_STATUS (tmMinor);

                break;

            case RDEVENTTV:


                /*
                 * Time is not latched in this routine - use RDEVENTTVREQ for
                 * latched time
                 */

                /* get minor time */
                TFP_IN_LONG (TFP_REG_EVENT0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time (unix secs (from 12am 1/1/70) */
                TFP_IN_LONG (TFP_REG_EVENT1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDEVENTTV):maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* translate to user values */

                pTv = (TFP_TV *) arg;

                /* unix secs (secs from 12am 1/1/70) */
                pTv->tv.tv_sec = tmMajor;

                /* usecs */
                pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTv->status = TFP_TM0_STATUS (tmMinor);


                break;

            case RDTIMETVREQ:
                if (intContext () == FALSE)
                    semTake (pDev->timemutex, WAIT_FOREVER);

                /* latch current minor/major time */
                TFP_OUT_LONG (TFP_REG_TIMEREQ (pDev->baseAdrsRegs), 0);

                /* get minor time */
                TFP_IN_LONG (TFP_REG_TIME0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time (unix secs (from 12am 1/1/70) */
                TFP_IN_LONG (TFP_REG_TIME1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDTIMETVREQ):maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* translate to user values */

                pTv = (TFP_TV *) arg;

                /* unix secs (secs from 12am 1/1/70) */
                pTv->tv.tv_sec = tmMajor;

                /* usecs */
                pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTv->status = TFP_TM0_STATUS (tmMinor);
                if (intContext () == FALSE)
                    semGive (pDev->timemutex);
                break;

            case RDEVENTTVREQ:
                if (intContext () == FALSE)
                    semTake (pDev->eventmutex, WAIT_FOREVER);

                /* latch current minor/major time (returned val not used) */
                TFP_IN_LONG (TFP_REG_EVENTREQ (pDev->baseAdrsRegs), &tmMinor);

                /* get minor time */
                TFP_IN_LONG (TFP_REG_EVENT0 (pDev->baseAdrsRegs), &tmMinor);

                /* get major time (unix secs (from 12am 1/1/70) */
                TFP_IN_LONG (TFP_REG_EVENT1 (pDev->baseAdrsRegs), &tmMajor);

                /* get minor time (usecs, hundreds nsecs, status) */
                TFP_DEBUG_MSG ("tfpIoctl(RDEVENTTVREQ):maj,min=x%08X,x%08X\n",
                               tmMajor, tmMinor, 0, 0, 0, 0);

                /* translate to user values */

                pTv = (TFP_TV *) arg;

                /* unix secs (secs from 12am 1/1/70) */
                pTv->tv.tv_sec = tmMajor;

                /* usecs */
                pTv->tv.tv_usec = TFP_TM0_BUSEC (tmMinor);

                /* hundreds of nsecs */
                pTv->hnsec = TFP_TM0_HNSEC (tmMinor);

                /* status */
                pTv->status = TFP_TM0_STATUS (tmMinor);
                if (intContext () == FALSE)
                    semGive (pDev->eventmutex);
                break;



            case SETSTROBETM:
                {
                    UINT32  tmp;

                    pTm = (TFP_TM *) arg;

                    TFP_IN_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), &tmp);
                    tmp = tmp & ~CTRL_STREN;
                    TFP_OUT_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), tmp);
                    TFP_OUT_LONG (TFP_REG_MAJSTRB (pDev->baseAdrsRegs), ((pTm->tm.tm_hour << 16) + (pTm->tm.tm_min << 8) + pTm->tm.tm_sec) & 0xffffff);
                    TFP_OUT_LONG (TFP_REG_MINSTRB (pDev->baseAdrsRegs), pTm->usec & 0xfffff);
                    TFP_IN_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), &tmp);
                    tmp = tmp | CTRL_STREN;
                    TFP_OUT_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), tmp);
                }

                break;

            case SETSTROBETV:
                {
                    UINT32  tmp;

                    pTv = (TFP_TV *) arg;

                    TFP_IN_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), &tmp);
                    tmp = tmp & ~CTRL_STREN;
                    TFP_OUT_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), tmp);
                    TFP_OUT_LONG (TFP_REG_MAJSTRB (pDev->baseAdrsRegs), pTv->tv.tv_sec);
                    TFP_OUT_LONG (TFP_REG_MINSTRB (pDev->baseAdrsRegs), pTv->tv.tv_usec & 0xfffff);
                    TFP_IN_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), &tmp);
                    tmp = tmp | CTRL_STREN;
                    TFP_OUT_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), tmp);
                }

                break;

            case SELTCFORMAT:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSetTcFmt (pDev, arg) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }

                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;


            case SELTCMOD:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSetTcm (pDev, arg) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;


                /* set 'decimal units time' */
            case SETTIMETM:

                pTm = (TFP_TM *) arg;

                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSetDecTime (pDev, pTm) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }

                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);

                break;


                /* set 'binary' time */
            case SETTIMETV:

                pTv = (TFP_TV *) arg;

                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSetBinTime (pDev, pTv) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }

                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);

                break;

                /* set year */
            case SETYEAR:

                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSetYr (pDev, arg) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }


                break;

            case SETPROPDELAY:  /* Set Proprgation Delay */


                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                if (tfpCmdSetDelay (pDev, arg) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;



            case SETPERIODIC:
                {
                    struct periodic *Per = (struct periodic *) arg;

                    if (intContext () == FALSE)
                        semTake (pDev->cmdmutex, WAIT_FOREVER);

                    TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETPERIODIC);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, Per->sync1pps);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 2, Per->n1 >> 8);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 3, Per->n1);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 4, Per->n2 >> 8);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 5, Per->n2);

                    TFP_DEBUG_MSG ("SETPERIODIC: pIn = 0x%x, 0x%x,0x%x,0x%x,0x%x,0x%x\n",
                                   TFP_IN_BYTE (pDev->baseAdrsInput),
                                   TFP_IN_BYTE (pDev->baseAdrsInput + 1),
                                   TFP_IN_BYTE (pDev->baseAdrsInput + 2),
                                   TFP_IN_BYTE (pDev->baseAdrsInput + 3),
                                   TFP_IN_BYTE (pDev->baseAdrsInput + 4),
                                   TFP_IN_BYTE (pDev->baseAdrsInput + 5));

                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG ("TFP SETPERIODIC Failed\n", 0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);

                }
                break;


            case SELFREQUENCYOUT:
                {
                    UINT32  tmp;

                    TFP_IN_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), &tmp);
                    switch (arg)
                    {
                        case FREQ_10MHZ:
                            tmp = (tmp & ~CTRL_FMASK) | CTRL_F10;
                            break;
                        case FREQ_5MHZ:
                            tmp = (tmp & ~CTRL_FMASK) | CTRL_F5;
                            break;
                        case FREQ_1MHZ:
                            tmp = (tmp & ~CTRL_FMASK) | CTRL_F1;
                            break;
                        default:
                            TFP_DEBUG_MSG ("TFP SELFREQUENCYOUT Failed - bad arg\n", 0, 0, 0, 0, 0, 0);
                            return (ERROR);
                    }
                    TFP_OUT_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), tmp);
                }
                break;

            case CONTROLEVENT:
                {
                    UINT32  tmp;

                    TFP_IN_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), &tmp);
                    switch (arg)
                    {
                        case EVENT_RISING:
                            tmp = tmp & ~CTRL_EVSENSE;
                            break;
                        case EVENT_FALLING:
                            tmp = tmp | CTRL_EVSENSE;
                            break;
                        case CAPLOCK_ENABLE:
                            tmp = tmp | CTRL_LOCKEN;
                            break;
                        case CAPLOCK_DISABLE:
                            tmp = tmp & ~CTRL_LOCKEN;
                            break;
                        case EVCAP_ENABLE:
                            tmp = tmp | CTRL_EVENTEN;
                            break;
                        case EVCAP_DISABLE:
                            tmp = tmp & ~CTRL_EVENTEN;
                            break;
                        case EVCAP_PERIODIC:
                            tmp = tmp | CTRL_EVSOURCE;
                            break;
                        case EVCAP_EVENT:
                            tmp = tmp & ~CTRL_EVSOURCE;
                            break;
                        default:
                            TFP_DEBUG_MSG ("TFP CONTROLEVENT Failed - bad arg\n", 0, 0, 0, 0, 0, 0);
                            return (ERROR);
                    }
                    TFP_OUT_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), tmp);
                }
                break;

            case CONTROLSTROBE:
                {
                    UINT32  tmp;

                    TFP_IN_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), &tmp);
                    switch (arg)
                    {
                        case STROBE_SECUS:
                            tmp = tmp & ~CTRL_STRMODE;
                            break;
                        case STROBE_USONLY:
                            tmp = tmp | CTRL_STRMODE;
                            break;
                        case STROBE_ENABLE:
                            tmp = tmp | CTRL_STREN;
                            break;
                        case STROBE_DISABLE:
                            tmp = tmp & ~CTRL_STREN;
                            break;
                        default:
                            TFP_DEBUG_MSG ("TFP CONTROLSTROBE Failed - bad arg\n", 0, 0, 0, 0, 0, 0);
                            return (ERROR);
                    }
                    TFP_OUT_LONG (TFP_REG_CONTROL (pDev->baseAdrsRegs), tmp);
                }
                break;

            case CAPUNLOCK:
                TFP_OUT_LONG (TFP_REG_UNLOCK (pDev->baseAdrsRegs), 0);
                break;

            case RDINTSTAT:
                TFP_IN_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), (UINT32 *) arg);
                break;

            case SETINTSTAT:
                TFP_OUT_LONG (TFP_REG_INTSTAT (pDev->baseAdrsRegs), arg);
                break;

            case RDINTMASK:
                TFP_IN_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), (UINT32 *) arg);
                break;

            case SETINTMASK:
                TFP_OUT_LONG (TFP_REG_INTMASK (pDev->baseAdrsRegs), arg);
                break;

#ifdef not_def
            case RDFIRMVERSION:
                {
                    struct gd_fwversion *Ver = (struct gd_fwversion *) arg;

                    if (intContext () == FALSE)
                        semTake (pDev->cmdmutex, WAIT_FOREVER);

                    TFP_OUT_BYTE (pDev->baseAdrsInput, COM_GETDATA);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, GETDATA_VERSION);

                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG ("TFP RDFIRMVERSION Failed\n", 0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }

                    Ver->vmajor = TFP_IN_BYTE (pDev->baseAdrsOutput + 1);
                    Ver->vminor = TFP_IN_BYTE (pDev->baseAdrsOutput + 2);
                    Ver->month = TFP_IN_BYTE (pDev->baseAdrsOutput + 3);
                    Ver->day = TFP_IN_BYTE (pDev->baseAdrsOutput + 4);
                    Ver->year = TFP_IN_BYTE (pDev->baseAdrsOutput + 5);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                }

                break;
#endif

            case CONTROLTIMEBASE:
                {
                    if (intContext () == FALSE)
                        semTake (pDev->cmdmutex, WAIT_FOREVER);
                    switch (arg)
                    {
                        case CLOCK_INTERNAL:
                            TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SELCLOCK);
                            TFP_OUT_BYTE (pDev->baseAdrsInput + 1, SELCLOCK_INT);

                            if (tfpCmdSend (pDev) == ERROR)
                            {
                                TFP_DEBUG_MSG ("TFP CONTROLTIMEBASE Failed\n", 0, 0, 0, 0, 0, 0);
                                if (intContext () == FALSE)
                                    semGive (pDev->cmdmutex);
                                return (ERROR);
                            }
                            break;
                        case CLOCK_EXTERNAL:
                            TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SELCLOCK);
                            TFP_OUT_BYTE (pDev->baseAdrsInput + 1, SELCLOCK_EXT);

                            if (tfpCmdSend (pDev) == ERROR)
                            {
                                TFP_DEBUG_MSG ("TFP CONTROLTIMEBASE Failed\n", 0, 0, 0, 0, 0, 0);
                                if (intContext () == FALSE)
                                    semGive (pDev->cmdmutex);
                                return (ERROR);
                            }

                            break;
                        case JAMSYNC_ENABLE:
                            TFP_OUT_BYTE (pDev->baseAdrsInput, COM_JAMSYNCENABLE);
                            TFP_OUT_BYTE (pDev->baseAdrsInput + 1, JAMSYNC_ENABLE);

                            if (tfpCmdSend (pDev) == ERROR)
                            {
                                TFP_DEBUG_MSG ("TFP CONTROLTIMEBASE Failed\n", 0, 0, 0, 0, 0, 0);
                                if (intContext () == FALSE)
                                    semGive (pDev->cmdmutex);
                                return (ERROR);
                            }

                            break;
                        case JAMSYNC_DISABLE:
                            TFP_OUT_BYTE (pDev->baseAdrsInput, COM_JAMSYNCENABLE);
                            TFP_OUT_BYTE (pDev->baseAdrsInput + 1, JAMSYNC_DISABLE);

                            if (tfpCmdSend (pDev) == ERROR)
                            {
                                TFP_DEBUG_MSG ("TFP CONTROLTIMEBASE Failed\n", 0, 0, 0, 0, 0, 0);
                                if (intContext () == FALSE)
                                    semGive (pDev->cmdmutex);
                                return (ERROR);
                            }
                            break;
                        case DISCIPLINE_ENABLE:
                            TFP_OUT_BYTE (pDev->baseAdrsInput, COM_DISCENABLE);
                            TFP_OUT_BYTE (pDev->baseAdrsInput + 1, DISC_EN);

                            if (tfpCmdSend (pDev) == ERROR)
                            {
                                TFP_DEBUG_MSG ("TFP CONTROLTIMEBASE Failed\n", 0, 0, 0, 0, 0, 0);
                                if (intContext () == FALSE)
                                    semGive (pDev->cmdmutex);
                                return (ERROR);
                            }

                            break;
                        case DISCIPLINE_DISABLE:
                            TFP_OUT_BYTE (pDev->baseAdrsInput, COM_DISCENABLE);
                            TFP_OUT_BYTE (pDev->baseAdrsInput + 1, DISC_DIS);

                            if (tfpCmdSend (pDev) == ERROR)
                            {
                                TFP_DEBUG_MSG ("TFP CONTROLTIMEBASE Failed\n", 0, 0, 0, 0, 0, 0);
                                if (intContext () == FALSE)
                                    semGive (pDev->cmdmutex);
                                return (ERROR);
                            }
                            break;
                        case FORCE_JAMSYNC:
                            TFP_OUT_BYTE (pDev->baseAdrsInput, COM_FORCEJAMSYNC);
                            if (tfpCmdSend (pDev) == ERROR)
                            {
                                TFP_DEBUG_MSG ("TFP CONTROLTIMEBASE Failed\n", 0, 0, 0, 0, 0, 0);
                                if (intContext () == FALSE)
                                    semGive (pDev->cmdmutex);
                                return (ERROR);
                            }

                            break;
                        case CLEAR_USEC:
                            break;
                        default:
                            errnoSet (EINVAL);
                            return (ERROR);
                    }
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                }
                break;

            case SETDAC:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETDAC);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg >> 8);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 2, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETDAC Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);

                break;


            case GETDATA:
                {
                    struct getdata_t *getdata = (struct getdata_t *) arg;
                    int     get_data_len = 0;
                    u_char *getdataptr = (u_char *) & getdata->data.tmode;
                    int     i;

                    if (intContext () == FALSE)
                        semTake (pDev->cmdmutex, WAIT_FOREVER);
                    TFP_OUT_BYTE (pDev->baseAdrsInput, COM_GETDATA);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, getdata->arg);

                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG ("TFP GETDATA Failed\n", 0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }

                    TFP_DEBUG_MSG ("TFP GETDATA arg = %x\n", getdata->arg, 0, 0, 0, 0, 0);
                    switch (getdata->arg)
                    {
                        case GETDATA_MODE:
                            get_data_len = sizeof (getdata->data.tmode);
                            break;
                        case GETDATA_TFORMAT:
                            get_data_len = sizeof (getdata->data.tformat);
                            break;
                        case GETDATA_YEAR:
                            get_data_len = sizeof (getdata->data.year);
                            break;
                        case GETDATA_PER:
                            get_data_len = sizeof (getdata->data.period);
                            break;
                        case GETDATA_TCFMT:
                            get_data_len = sizeof (getdata->data.tcformat);
                            break;
                        case GETDATA_TCMOD:
                            get_data_len = sizeof (getdata->data.tcmod);
                            break;
                        case GETDATA_TOFF:
                            get_data_len = sizeof (getdata->data.offset);
                            break;
                        case GETDATA_UTCINFO:
                            get_data_len = sizeof (getdata->data.utc);
                            break;

                        case GETDATA_TCOUTFMT:
                            get_data_len = sizeof (getdata->data.tcoutfmt);
                            break;

                        case GETDATA_TCGENOFF:
                            get_data_len = sizeof (getdata->data.genoff);
                            break;

                        case GETDATA_LOCTMOFF:
                            get_data_len = sizeof (getdata->data.locoff);
                            break;

                        case GETDATA_LEAPSEC:
                            get_data_len = sizeof (getdata->data.leapsec);
                            break;


                        case GETDATA_FWVER:
                            get_data_len = sizeof (getdata->data.version);
                            break;
                        case GETDATA_CLKSRC:
                            get_data_len = sizeof (getdata->data.clksrc);
                            break;
                        case GETDATA_JAMSC:
                            get_data_len = sizeof (getdata->data.jsctl);
                            break;
                        case GETDATA_OSCCTL:
                            get_data_len = sizeof (getdata->data.ocsctl);
                            break;
                        case GETDATA_DAVAL:
                            get_data_len = sizeof (getdata->data.davalue);
                            break;

                        case GETDATA_BATTSTAT:
                            get_data_len = sizeof (getdata->data.battstat);
                            break;

                        case GETDATA_CLKVAL:
                            get_data_len = sizeof (getdata->data.clkval);
                            break;

                        case GETDATA_DTFW:
                            get_data_len = sizeof (getdata->data.dtfw);
                            break;

                        case GETDATA_ASSEMB:
                            get_data_len = sizeof (getdata->data.assemb);
                            break;

                        case GETDATA_HWFAB:
                            get_data_len = sizeof (getdata->data.hwfab);
                            break;

                        case GETDATA_TFPMODEL:
                            get_data_len = sizeof (getdata->data.tfpm);
                            break;

                        case GETDATA_SERIAL:
                            get_data_len = sizeof (getdata->data.serialnum);
                            break;

                        default:
                            get_data_len = 0;
                            if (intContext () == FALSE)
                                semGive (pDev->cmdmutex);
                            errnoSet (EINVAL);
                            return (ERROR);

                    }

                    for (i = 0; i < get_data_len; i++)
                    {
                        getdataptr[i] = TFP_IN_BYTE (pDev->baseAdrsOutput + 1 + i);
                        TFP_DEBUG_MSG ("TFP GETDATA [%d] = %x\n",
                                       i, getdataptr[i], 0, 0, 0, 0);
                    }
                    TFP_DEBUG_MSG ("TFP GETDATA Complete\n", 0, 0, 0, 0, 0, 0);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SOFTWARERESET:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);

                if (tfpCmdSoftwareReset (pDev) == ERROR)
                {
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }

                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETTCOUTFMT:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTCOUTFMT);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETTCOUTFMT Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETGENTMOFFSET:
                {
                    struct tcgenoffset *tcgoff = (struct tcgenoffset *) arg;

                    if (intContext () == FALSE)
                        semTake (pDev->cmdmutex, WAIT_FOREVER);
                    TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETGENTMOFF);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, tcgoff->offset >> 8);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 2, tcgoff->offset);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 3, tcgoff->half_hour);

                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG ("TFP SETGENTMOFFSET Failed\n", 0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }
                }

                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETLOCTMOFFSET:
                {
                    struct loctmoffset *locoff = (struct loctmoffset *) arg;

                    if (intContext () == FALSE)
                        semTake (pDev->cmdmutex, WAIT_FOREVER);

                    TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETLOCTMOFF);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, locoff->offset >> 8);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 2, locoff->offset);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 3, locoff->half_hour);

                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG ("TFP SETLOCTMOFFSET Failed\n", 0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                }
                break;

            case SETLEAPSECEVENT:
                {
                    struct leapseconds *lpsec = (struct leapseconds *) arg;

                    if (intContext () == FALSE)
                        semTake (pDev->cmdmutex, WAIT_FOREVER);
                    TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETLEAPSEC);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 1, lpsec->ls_flag);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 2, lpsec->leap_tm >> 24);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 3, lpsec->leap_tm >> 16);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 4, lpsec->leap_tm >> 8);
                    TFP_OUT_BYTE (pDev->baseAdrsInput + 5, lpsec->leap_tm);

                    if (tfpCmdSend (pDev) == ERROR)
                    {
                        TFP_DEBUG_MSG ("TFP SETLEAPSECEVENT Failed\n", 0, 0, 0, 0, 0, 0);
                        if (intContext () == FALSE)
                            semGive (pDev->cmdmutex);
                        return (ERROR);
                    }

                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                }
                break;

            case SETDISCGAIN:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETDISCGAIN);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg >> 8);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 2, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETDISGAIN Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETCLKVAL:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETCLKVAL);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg >> 24);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 2, arg >> 16);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 3, arg >> 8);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 4, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETCLKVAL Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETGPSTMFMT:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_GPSTIMEFORMAT);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg >> 24);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 2, arg >> 16);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 3, arg >> 8);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 4, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETGPSTMFMT Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }

                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETGPSMDFLG:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETGPSMDFLG);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETGPSMDFLG Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETLOCTMFLG:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETLOCTMFLG);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETLOCTMFLG Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }

                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SETYRINCFLG:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETYRINCFLG);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, arg);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SETYRINCFLG Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }


                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case SYNCRTC:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SYNCRTC);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP SYNCRTC Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;

            case DISCBATT:
                if (intContext () == FALSE)
                    semTake (pDev->cmdmutex, WAIT_FOREVER);
                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_DISCBATTRTC);

                if (tfpCmdSend (pDev) == ERROR)
                {
                    TFP_DEBUG_MSG ("TFP DISCBATT Failed\n", 0, 0, 0, 0, 0, 0);
                    if (intContext () == FALSE)
                        semGive (pDev->cmdmutex);
                    return (ERROR);
                }
                if (intContext () == FALSE)
                    semGive (pDev->cmdmutex);
                break;


            default:

                errnoSet (EINVAL);
                return (ERROR);

        }                       /* switch */

        return (OK);

    }

/******************************************************************************
*
* tfpCmdSetTmMode - command tfp to set 'timing mode'.
*
* The 'mode' argument must be set to the timing mode desired
* and must be one of the following:
*       define MODE_TIMECODE            0
*       define MODE_FREERUN             1
*       define MODE_EXT1PPS             2
*       define MODE_RTC                 3
*       define MODE_GPS                 6
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdSetTmMode
            (
                     TFP_DEV * pDev,
                     int mode
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIMINGMODE);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) mode);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetTmMode: timed out waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

                TFP_DEBUG_MSG ("tfpCmdSetTmMode: cmd complete, mode=%d\n",
                                       mode, 0, 0, 0, 0, 0);

        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetTmFmt - command tfp to set 'time register format'
*
* The 'fmt' argument must be set to one of the following:
*       define TIME_DECIMAL     0
*       define TIME_BINARY      1
*
* Upon return, the user must set/read 'major time' according to the format set.
*
* NOTE: In the Programmer's Guide, the term 'binary time' is used to refer
*       to the time registers when they are formatted with unix time.
*       By unix time, they mean 'seconds since 1970'.
*
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdSetTmFmt
            (
                     TFP_DEV * pDev,
                     int fmt
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIMEFORMAT);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) fmt);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetTmFmt: timeout waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

                TFP_DEBUG_MSG ("tfpCmdSetTmFmt: cmd complete, fmt=%02X\n",
                                       fmt, 0, 0, 0, 0, 0);

        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetTcFmt - command tfp to set 'timecode format'
*
* The 'fmt' argument must be set to one of the following:
#       define TC_IRIGA         'A'
#       define TC_IRIGB         'B'
#       define TC_IEEE          'I'

* RETURNS: OK always.
*/

    STATUS  tfpCmdSetTcFmt
            (
                     TFP_DEV * pDev,
                     int fmt
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTCFORMAT);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) fmt);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetTcFmt: timeout waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

                TFP_DEBUG_MSG ("tfpCmdSetTcFmt: cmd complete, fmt=%02X\n",
                                       fmt, 0, 0, 0, 0, 0);

        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetBinTime - set major time using 'binary' format
*
* The value passed is 'seconds from Jan 1, 1970'
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdSetBinTime
            (
                     TFP_DEV * pDev,
                     TFP_TV * pTv
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIME);

        /* msb #1 */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1,
                      MSB(MSW((UINT32) pTv->tv.tv_sec)));
        /* msb #2 */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 2,
                      LSB(MSW((UINT32) pTv->tv.tv_sec)));
        /* lsb #1 */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 3,
                      MSB(LSW((UINT32) pTv->tv.tv_sec)));
        /* lsb #2 */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 4,
                      LSB(LSW((UINT32) pTv->tv.tv_sec)));

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetBinTime: timeout waiting for cmd interface\n",
                                       0, 0, 0, 0, 0, 0);
            return (ERROR);
        }


        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetDecTime - set major time using 'decimal' format
*
* The following members of the 'tm' user struct are used:
*    tm_year: 1970 - 2050
*    tm_yday: 0 - 366
*    tm_hour: 0 - 23
*    tm_min: 0 - 59
*    tm_sec: 0 - 59
* Other members are not used.
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdSetDecTime
            (
                     TFP_DEV * pDev,
                     TFP_TM * pTm
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIME);

        /* msb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, MSB ((UINT16) pTm->tm.tm_year));
        /* lsb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 2, LSB ((UINT16) pTm->tm.tm_year));

        /* msb - yday */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 3, MSB ((UINT16) pTm->tm.tm_yday));
        /* lsb - yday */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 4, LSB ((UINT16) pTm->tm.tm_yday));

        /* lsb - hour (0-23) */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 5, LSB ((UINT16) pTm->tm.tm_hour));

        /* lsb - min (0-59) */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 6, LSB ((UINT16) pTm->tm.tm_min));

        /* lsb - sec (0-59) */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 7, LSB ((UINT16) pTm->tm.tm_sec));


        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetDecTime: timeout waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

                TFP_DEBUG_MSG ("tfpCmdSetDecTime: cmd complete\n", 0, 0, 0, 0, 0, 0);

        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetYr - set year
*
* Valid range for 'year' is 1970 - 2050
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdSetYr
            (
                     TFP_DEV * pDev,
                     int year
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETYEAR);

        /* msb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, MSB ((UINT16) year));
        /* lsb - year */
        TFP_OUT_BYTE (pDev->baseAdrsInput + 2, LSB ((UINT16) year));

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetYr: timeout waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }
                /* delay so on-card cmd may actually complete */
                (void) taskDelay (sysClkRateGet () / 5); 

                TFP_DEBUG_MSG ("tfpCmdSetYr: cmd complete, year=%d\n", year, 0, 0, 0, 0, 0);

        return (OK);

    }


/******************************************************************************
*
* tfpSoftwareReset - perform software reset cmd
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdSoftwareReset
            (
                     TFP_DEV * pDev
    )
    {
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SOFTRESET);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, SW_RESET);

        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("TFP SOFTWARERESET Failed\n", 0, 0, 0, 0, 0, 0);
            return (ERROR);
        }
        taskDelay (sysClkRateGet ());

        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetTcm - command tfp to set 'timecode modulation format'
*
* The 'type' argument must be set to one of the following modulation types:
*    TFP_TCM_AM      0x4D    * expect 'amplitude modulated' format
*    TFP_TCM_PCM     0x44    * expect 'pulse code modulated' format

* RETURNS: OK always.
*/

    STATUS  tfpCmdSetTcm
            (
                     TFP_DEV * pDev,
                     int type
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTCMOD);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) type);

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetTcm: timeout waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

                TFP_DEBUG_MSG ("tfpCmdSetTcm: cmd complete, type=%02X\n",
                                       type, 0, 0, 0, 0, 0);

        return (OK);

    }


/******************************************************************************
*
* tfpCmdSetDelay - command tfp to set 'propagation delay'
*
* The 'offset' argument is in units of 100ns and may be positive
* or negative.

* RETURNS: OK always.
*/

    STATUS  tfpCmdSetDelay
            (
                     TFP_DEV * pDev,
                     int offset
    )
    {

        /* stuff cmd arg bytes in tfp input area */
        TFP_OUT_BYTE (pDev->baseAdrsInput, COM_SETTIMINGOFFSET);
        TFP_OUT_BYTE (pDev->baseAdrsInput + 1, (UINT8) LLSB (offset));
        TFP_OUT_BYTE (pDev->baseAdrsInput + 2, (UINT8) LNLSB (offset));
        TFP_OUT_BYTE (pDev->baseAdrsInput + 3, (UINT8) LNMSB (offset));
        TFP_OUT_BYTE (pDev->baseAdrsInput + 4, (UINT8) LMSB (offset));

        /* tell tfp to execute and complete command */
        if (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSetDelay: timeout waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

                TFP_DEBUG_MSG ("tfpCmdSetDelay: cmd complete, offset=%02X\n",
                                       offset, 0, 0, 0, 0, 0);

        return (OK);

    }



/******************************************************************************
*
* tfpCmdGetTfpModel - command tfp to return 'tfp model' characters
*
* The model chars are TFP_MODEL_LEN(9) long and can be treated
* as an unterminated string. The 'model' chars are placed in
* caller's 'buf' arg as a terminated string.
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdGetTfpModel
            (
                     TFP_DEV * pDev,
                     char *buf
    )
    {

        if (tfpCmdReqDataSend (pDev, COM_REQTFPMODEL) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdTfpModel: tfpCmdReqDataSend failed\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

        /* get string (skip over request code in first location) */
        tfpGetOutBytes (pDev->baseAdrsOutput + 1, buf, TFP_MODEL_LEN);

        /* string is unterminated, so terminate it */
        *(buf + TFP_MODEL_LEN) = EOS;

        TFP_DEBUG_MSG ("tfpCmdTfpModel: str=%s\n", (int) buf, 0, 0, 0, 0, 0);

        return (OK);

    }


/******************************************************************************
*
* tfpCmdReqDataSend - command tfp to exexute a 'data request' type of command
*
* This function is used to send a 'tfp request data (cmd id=0x19)' command.
* The 'reqDataCode' arg identifies the specific data that the tfp is to return.
*
* RETURNS: OK always.
*/

    STATUS  tfpCmdReqDataSend
            (
                     TFP_DEV * pDev,
                     UINT8 reqDataCode
    )
    {
        UINT8   val;


        /* issue 'req data' cmd */

                TFP_OUT_BYTE (pDev->baseAdrsInput, COM_GETDATA);
                TFP_OUT_BYTE (pDev->baseAdrsInput + 1, reqDataCode);

        if      (tfpCmdSend (pDev) == ERROR)
        {
            TFP_DEBUG_MSG ("tfpCmdSendReqData: timeout waiting for cmd interface\n",
                           0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

        /* verify request code located in first location */
                val = TFP_IN_BYTE (pDev->baseAdrsOutput);

        if (val != reqDataCode)
        {
            TFP_DEBUG_MSG ("tfpCmdSendReqData: bad req code read\n", 0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

        return (OK);

    }



/******************************************************************************
*
* tfpCmdSend - send command to TFP for execution
*
* The caller is to fill the 'input area' with a set of command
* byte values for some tfp command. This function is then called
* to perform the command execution protocol. Upon return, the
* command is complete.
*
* RETURNS: OK always.
*/

    STATUS
        tfpCmdSend (TFP_DEV * pDev)
    {
        ULONG   val;
        FAST int i = 0;
        const   secsTimeOut = sysClkRateGet () * 5;


#ifdef notdef
        /* clear bit 0 of 'ack reg by writing '1' to reg */
        TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_CLR);

        /* inform tfp of command is waiting by writing 0x80 to 'ack' reg */
        TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_CMDRDY);
#endif
        TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), 0x81);

        /* wait for tfp to set bit 0 of 'ack' reg */
        while (i++ < secsTimeOut)
        {
            TFP_IN_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), &val);

            if ((val & TFP_ACK_COMPLETE) > 0)
                break;

            (void) taskDelay (1);

        }

        /* clear bit 0 of 'ack reg by writing '1' to reg */
        TFP_OUT_LONG (TFP_REG_ACK (pDev->baseAdrsRegs), TFP_ACK_CLR);

        if (i == secsTimeOut)
        {
            TFP_DEBUG_MSG ("tfpCmdSend: timeout\n", 0, 0, 0, 0, 0, 0);
            return (ERROR);
        }

        return (OK);

    }



/******************************************************************************
*
* tfpGetOutBytes - copy tfp 'output area' bytes to local address
*
* This routine copies bytes the tfp has returned to the 'ouput area'
* as the result of some 'data request' command to a local address.
*
* outputAdrs = the address as seen from the cpu of the tfp 'output area'
*              to start copying from.
*
* pDest = the address to copy 'output area' bytes to.
*
* len = the number of bytes to copy.
*
* RETURNS: number of bytes copied (should always be 'len')
*/

    int     tfpGetOutBytes
            (
                     UINT32 outputAdrs,
                     u_char * pDest,
                     int len
    )
    {
        u_char *p = pDest;
        UINT32  adrs = outputAdrs;
        int     i;

        for     (i = 0; i < len; i++)
                   *p++ = TFP_IN_BYTE (adrs++);

                return (len);

    }

#ifdef __cplusplus
}

#endif

#endif                          /* __INCbc635c */
