
/*
 sample.c  - This is an interactive test program demonstrating the
             functionality of the bc635/637 VxWorks Device Driver.

             It is intended as example code, to assist a programmer in
             developing their own code using the driver.

             This test program lacks proper error checking, and also 
             does not prevent erroneous actions.  (An example of an
             erroneous action would be to read a decimal time format when
             the board is in binary time mode).
*/
/*
  *******************************************************************************
** Copyright (C) 2000 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/

/**************************************************************************************/

#include "vxWorks.h"
#include "stdio.h"
#include "logLib.h"
#include "taskLib.h"
#include "ioLib.h"
#include "errnoLib.h"
#include "sysLib.h"
#include "usrLib.h"
#include "selectLib.h"
#include "ctype.h"
#include "string.h"
#include "bc635.h"
#include "sysLib.h"

#define RC_OK           1
#define RC_ERROR   -1

/**************************************************************************************/
/* Global Variables */
/**************************************************************************************/
int     fd;
struct getdata_t get;

/* Forward declarations */

int     menu_main ();


/**************************************************************************************/
/* Description : main */
/**************************************************************************************/
int 
sample ()
{

    if ((fd = open ("/tfp0", 0, 0)) == ERROR)   /* open bc635pci device */
    {
        printf ("Error opening Device Driver ......... Exiting");
        printErrno (errnoGet ());
        return (ERROR);
    }

    return menu_main ();

}


/**************************************************************************************/
/* Description: Set Oper Mode */
/**************************************************************************************/
void 
pci_mode ()
{
    int     oper_mode = 0;

    printf ("\n\n\n\t    Select Operational Mode:\n\n");
    printf ("\t0. Time Code Mode\n");
    printf ("\t1. Free Running Mode\n");
    printf ("\t2. External 1PPS\n");
    printf ("\t3. Real Time Clock Mode\n");
    printf ("\t4. GPS Mode\n\n");
    printf ("\t   Select: ");
    scanf ("%i", &oper_mode);

    switch (oper_mode)
    {
        case 0:
            ioctl (fd, SELTIMINGMODE, (int) MODE_TIMECODE);     /* Timing Mode: time
                                                                 * code */
            break;
        case 1:
            ioctl (fd, SELTIMINGMODE, (int) MODE_FREERUN);      /* Timing Mode: Free
                                                                 * Running */
            break;
        case 2:
            ioctl (fd, SELTIMINGMODE, (int) MODE_EXT1PPS);      /* Timing Mode: 1PPS */
            break;
        case 3:
            ioctl (fd, SELTIMINGMODE, (int) MODE_RTC);  /* Timing Mode: RTC */
            break;
        case 4:
            ioctl (fd, SELTIMINGMODE, (int) MODE_GPS);  /* Timing Mode: GPS */
            break;
        default:
            printf ("Error setting opertional mode!!!!");
            break;
    }
}

/**************************************************************************************/
/* Description : Set Time Code Format */
/**************************************************************************************/
void 
pci_time_code ()
{
    int     tc_fmt;
    int     tc_mod;

    printf ("\n\n\n\t    Select Time Code Format:\n\n");
    printf ("\t1. IRIG A\n");
    printf ("\t2. IRIG B\n\n");
    printf ("\t   Select: ");
    scanf ("%d", &tc_fmt);
    switch (tc_fmt)
    {
        case 1:
            ioctl (fd, SELTCFORMAT, TC_IRIGA);  /* IRIG A time code */
            break;
        case 2:
            ioctl (fd, SELTCFORMAT, TC_IRIGB);  /* IRIG B time code */
            break;
        default:
            printf ("Error Setting Time Code Format!");
            break;
    }

    printf ("\n\n\n\t    Select Time Code Modulation:\n\n");
    printf ("\t1. Modulated\n");
    printf ("\t2. DC Level Shift\n\n");
    printf ("\t   Select: ");
    scanf ("%d", &tc_mod);
    switch (tc_mod)
    {
        case 1:
            ioctl (fd, SELTCMOD, MOD_AM);       /* Modulated */
            break;
        case 2:
            ioctl (fd, SELTCMOD, MOD_DCLS);     /* DCLS */
            break;
        default:
            printf ("Error Setting Time Code Modulation!");
            break;
    }
}

/**************************************************************************************/
/* Description : Set Output Frequency */
/**************************************************************************************/
void 
pci_out_freq ()
{
    int     out_freq = 3;

    printf ("\n\n\n\t    Select Output Frequency:\n\n");
    printf ("\t1. 1  MHz\n");
    printf ("\t2. 5  MHz\n");
    printf ("\t3. 10 MHz\n\n");
    printf ("\t   Select: ");
    scanf ("%d", &out_freq);
    switch (out_freq)
    {
        case 1:
            ioctl (fd, SELFREQUENCYOUT, FREQ_1MHZ);     /* 1 MHz */
            break;
        case 2:
            ioctl (fd, SELFREQUENCYOUT, FREQ_5MHZ);     /* 5 MHz */
            break;
        case 3:
            ioctl (fd, SELFREQUENCYOUT, FREQ_10MHZ);    /* 10 MHz */
            break;
        default:
            printf ("Error Setting output frequency!");
            break;
    }
}

/**************************************************************************************/
/* Description : Set Time Format */
/**************************************************************************************/
void 
pci_time_format ()
{
    int     time_format = 0;

    printf ("Select Time Format\n  0. Decimal\n  1. Binary\n\nSelect: ");
    scanf ("%d", &time_format);

    switch (time_format)
    {
        case 0:
            ioctl (fd, SELTIMEFORMAT, TIME_DECIMAL);    /* decimal time format */
            break;
        case 1:
            ioctl (fd, SELTIMEFORMAT, TIME_BINARY);     /* Binary time format */
            break;
        default:
            printf ("\nError Setting Time Format!");
            break;
    }
}

/**************************************************************************************/
/* Description : Set Heartbeat Counters */
/**************************************************************************************/
void 
pci_heartbeat ()
{
    struct periodic sper;
    int     hrt_mode;
    int     counter1 = 10;
    int     counter2 = 10;

    sper.sync1pps = PERIODIC_SYNC;

    printf ("\n\n\n\t    Heartbeat Counters and Mode:\n\n");
    printf ("\t1. Synchronous mode\n");
    printf ("\t2. Asynchronous mode\n\n");
    printf ("\t   Select: ");
    scanf ("%i", &hrt_mode);
    printf ("\n\tEnter 4 hex value for counetr1: \n");
    scanf ("%i", &counter1);
    printf ("\n\tEnter 4 hex value for counetr2: \n");
    scanf ("%i", &counter2);

    if (hrt_mode == 2)
        sper.sync1pps = PERIODIC_NOSYNC;
    if (counter1 >= 2 && counter1 <= 65535)
        sper.n1 = counter1;
    if (counter2 >= 2 && counter2 <= 65535)
        sper.n2 = counter2;

    ioctl (fd, SETPERIODIC, (int) &sper);
}


/**************************************************************************************/
/* Description : Set Time */
/**************************************************************************************/
void 
pci_set_time ()
{
    int     choice;
    int     year = 1999;
    TFP_TM  tm;
    TFP_TV  tv;
    time_t tUsr_secs;
    time_t t1970_secs;
    struct tm tmpTm;

    printf ("\n\n\n\t Set Time Functions :\n\n");
    printf ("\t1. Set Year\n");
    printf ("\t2. Set Time TM\n");
    printf ("\t3. Set Time TV\n");
    printf ("\t   Select: ");

    scanf ("%d", &choice);
   
    switch (choice)
    {
       case 1: 
           printf ("\n\tEnter Year (1990 to 2050): \n");
           scanf ("%d", &year);
           if (year >= 1990 && year <= 2050)
              ioctl (fd, SETYEAR, year);      /* Set year */
           else
              printf ("\nError: Value Out of Range");
           break;

       case 2:

           printf ("\n\tEnter Year [1970-2036]: ");
           scanf ("%d", &(tm.tm.tm_year));
           printf ("\tEnter Year Day [0-366]: ");
           scanf ("%d", &tm.tm.tm_yday);
           printf ("\t     Enter Hour [0-23]: ");
           scanf ("%d", &tm.tm.tm_hour);
           printf ("\t      Enter Min [0-59]: ");
           scanf ("%d", &tm.tm.tm_min);
           printf ("\t      Enter Sec [0-59]: ");
           scanf ("%d", &tm.tm.tm_sec);
      
           ioctl (fd, SETTIMETM, (int)&tm);

           break;
  
       case 3:

           /*
            * We will prompt the user for year,month,day info
            * and then use the vxWorks mktime() routine
            * to generate a 'binary' time of 'seconds from 1970'.
            * But, vxWorks apparently calculates this based on
            * 'seconds from 1900' (see time.h). So, we will
            * calculate 'seconds from 1900 to 1970' and
            * 'seconds from 1900 to user's date' and then subtract two.
            */ 

           /* prompt user for time/date info */
           printf ("\n\tEnter Year [1970-2036]: ");
           scanf ("%d", &(tmpTm.tm_year));
           printf ("\t    Enter Month [0-11]: ");
           scanf ("%d", &tmpTm.tm_mon);
           printf ("\tEnter Month Day [1-31]: ");
           scanf ("%d", &tmpTm.tm_mday);
           printf ("\t     Enter Hour [0-23]: ");
           scanf ("%d", &tmpTm.tm_hour);
           printf ("\t      Enter Min [0-59]: ");
           scanf ("%d", &tmpTm.tm_min);
           printf ("\t      Enter Sec [0-59]: ");
           scanf ("%d", &tmpTm.tm_sec);

           tmpTm.tm_yday = 0;
           tmpTm.tm_wday = 0;
           tmpTm.tm_isdst = 0;

           /* generate '1900 to user-date seconds' */
           tUsr_secs = mktime (&tmpTm);


           /* now get 1970 seconds */

           tmpTm.tm_sec = 0;
           tmpTm.tm_min = 0;
           tmpTm.tm_hour = 0;
           tmpTm.tm_mday = 1;
           tmpTm.tm_mon = 0;
           tmpTm.tm_wday = 0;
           tmpTm.tm_yday = 0;
           tmpTm.tm_isdst = 0;
           tmpTm.tm_year = 1970;

           /* generate '1900 to 1970 seconds' */
           t1970_secs = mktime (&tmpTm);

           /* now generate '1970 to user-date seconds' */
           tv.tv.tv_sec = tUsr_secs - t1970_secs;

           ioctl (fd, SETTIMETV, (int)&tv);

           break;

       default:
           break;
    }

}


/**************************************************************************************/
/* Description : Request Current Settings */
/**************************************************************************************/
void 
pci_req_current_settings ()
{
    const char smode[7][10] = {"Time Code", "Free Run", "Ext 1PPS", "RTC", "Reserved", "Reserved", "GPS"};
    const char stm_fmt[2][10] = {"Decimal", "Binary"};
    int     tm_fmt = 0;
    int     year = 0;
    int     dac_val = 0;
    int     timing_off = 0;
    INT16   gen_off = 0;
    INT16   loc_off = 0;
    u_char  tc_fmt = ' ';
    u_char  tc_mod = ' ';
    u_char  clk_scr = ' ';
    UINT8   mode = 0;

    get.arg = GETDATA_MODE;     /* Timing Mode */
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        mode = (int) get.data.tmode;
    printf ("\nMode\t\t: %s", smode[mode]);

    get.arg = GETDATA_TFORMAT;  /* Time Format */
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        tm_fmt = (int) get.data.tformat;
    printf ("\nTime Format\t: %s", stm_fmt[tm_fmt]);

    get.arg = GETDATA_YEAR;     /* Year */
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        year = (int) get.data.year;
    printf ("\nYear\t\t: %d", year);

    get.arg = GETDATA_PER;
    ioctl (fd, GETDATA, (int)&get);
    printf ("\nPeriodic Output\t: sync1pps=%x  n1=%x, n2=%x",
       get.data.period.sync1pps, get.data.period.n1, 
       get.data.period.n2);

    get.arg = GETDATA_TCFMT;    /* Time Code Format */
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        tc_fmt = (u_char) get.data.tcformat;
    printf ("\nTime Code\t: IRIG %c", tc_fmt);

    get.arg = GETDATA_TCMOD;    /* Time Code Modulation */
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        tc_mod = (u_char) get.data.tcmod;
    printf ("\nCode Modulation\t: %c", tc_mod);

    get.arg = GETDATA_TOFF;
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        timing_off = (int) get.data.offset;
    printf ("\nTiming Offset\t: %d", timing_off);

    get.arg = GETDATA_UTCINFO;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nUTC\t : enable=%d,leapsec=%d,leapevt=%d,evttime=%x", get.data.utc.enable, get.data.utc.leapsec,
       get.data.utc.leapevt, get.data.utc.evttime);

    get.arg = GETDATA_TCOUTFMT;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nTCOUTFMT\t: %d", get.data.tcoutfmt);

    get.arg = GETDATA_LOCTMOFF;
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
         loc_off = get.data.locoff.offset;
    printf ("\nLoc Time Offset\t: %d", loc_off);

    get.arg = GETDATA_TCGENOFF;
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
         gen_off = get.data.genoff.offset;
    printf ("\nGen Time Offset\t: %d", gen_off);

    get.arg = GETDATA_LEAPSEC;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nLEAPSEC: ls_flag=%d, leap_tm=%d", 
       get.data.leapsec.ls_flag, get.data.leapsec.leap_tm);
   
    get.arg = GETDATA_FWVER;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nFWVER\t: vmajor=%d, vminor=%d, month=%d, day=%d, year=%d",
       get.data.version.vmajor, get.data.version.vminor, 
       get.data.version.month, get.data.version.day,
       get.data.version.year); 


    get.arg = GETDATA_CLKSRC;
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        clk_scr = (u_char) get.data.clksrc;
    if (clk_scr == 'I')
        printf ("\nOscillator  \t: Internal");
    else
        printf ("\nOscillator  \t: External");
 

    get.arg = GETDATA_DAVAL;
    if (!((ioctl (fd, GETDATA, (int) &get)) < 0))
        dac_val = (int) get.data.davalue;
    printf ("\nDAC Value\t: %x", dac_val);

    get.arg = GETDATA_JAMSC;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nJAMSC\t\t: %d", get.data.jsctl);

    get.arg = GETDATA_OSCCTL;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nOCSCTL\t: discctl=%d, phasectl=%d, phasestep=%d, gain=%x, con=%x", 
       get.data.ocsctl.discctl, get.data.ocsctl.phasectl,
       get.data.ocsctl.phasestep, get.data.ocsctl.gain,
       get.data.ocsctl.con);

    get.arg = GETDATA_BATTSTAT;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nBATSTATT\t: %x", get.data.battstat);

    get.arg= GETDATA_CLKVAL;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nCLKVAL\t\t: %x", get.data.clkval);

    get.arg = GETDATA_DTFW;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nDTFW\t\t: %c%c%c%c%c%c%c%c%c%c%c",
      get.data.dtfw.dt01,
      get.data.dtfw.dt02,
      get.data.dtfw.dt03,
      get.data.dtfw.dt04,
      get.data.dtfw.dt05,
      get.data.dtfw.dt06,
      get.data.dtfw.dt07,
      get.data.dtfw.dt08,
      get.data.dtfw.dt09,
      get.data.dtfw.dt10,
      get.data.dtfw.dt11);

    get.arg = GETDATA_ASSEMB;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nASSEMB\t\t: %x", get.data.assemb);

    get.arg = GETDATA_TFPMODEL;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nTFPMODEL\t: %c%c%c%c%c%c%c%c", 
      get.data.tfpm.tfp1,
      get.data.tfpm.tfp2,
      get.data.tfpm.tfp3,
      get.data.tfpm.tfp4,
      get.data.tfpm.tfp5,
      get.data.tfpm.tfp6,
      get.data.tfpm.tfp7,
      get.data.tfpm.tfp8);

    get.arg = GETDATA_SERIAL;
    ioctl (fd, GETDATA, (int) &get);
    printf ("\nSERIAL Number\t: %x", get.data.serialnum); 
  
    


    return;

}

/***************************************************************************
* tmPrint - print 'tm' struct
*
* RETURNS: OK
*/

LOCAL STATUS tmPrint
        (
                 TFP_TM * pTm
)

{

    (void) printf ("\n   tm_sec: %d\n", pTm->tm.tm_sec);
    (void) printf ("   tm_min: %d\n", pTm->tm.tm_min);
    (void) printf ("  tm_hour: %d\n", pTm->tm.tm_hour);
    (void) printf ("  tm_yday: %d\n", pTm->tm.tm_yday);
    (void) printf ("  tm_year: %d\n", pTm->tm.tm_year);
    (void) printf ("     usec: %d\n", pTm->usec);
    (void) printf ("    hnsec: %d\n", pTm->hnsec);
    (void) printf ("   status: x%01X\n", pTm->status);

    return (OK);
}

/***************************************************************************
* tvPrint - print 'tv' struct
*
* RETURNS: OK
*/
LOCAL STATUS tvPrint
        (
                 TFP_TV * pTv
)

{
    struct tm tm;
    char tmbuf[80];
    size_t bufLen = sizeof(tmbuf);

    (void) printf ("   tv_sec: %lu (x%08X)\n",
                   pTv->tv.tv_sec, (int) pTv->tv.tv_sec);
    (void) printf ("     usec: x%ld\n", pTv->tv.tv_usec);
    (void) printf ("    hnsec: x%x\n", pTv->hnsec);
    (void) printf ("   status: x%01X\n", pTv->status);

    gmtime_r (&pTv->tv.tv_sec, &tm);
    
    asctime_r (&tm, tmbuf, &bufLen);

    /* display date/time */
    (void) printf ("time/date: %s\n", tmbuf);


    return (OK);
}

/**************************************************************************************/
/* Description : read_time_menu */
/**************************************************************************************/

void 
read_time_menu ()
{
    int     choice;
    int     ioctl_choice;
    int     arg_choice;
    int     print_choice;
    TFP_TM  tfpTM;
    TFP_TV  tfpTV;


    printf ("\n\n\n  Read Time Formats:\n\n");
    printf ("\t1. RDTIMETM (Unlatched, DECIMAL)\n");
    printf ("\t2. RDTIMETMREQ (Latched, DECIMAL)\n");
    printf ("\t3. RDTIMETV (Unlatched, BINARY)\n");
    printf ("\t4. RDTIMETVREQ (Latched, BINARY)\n\n");
    printf ("\t5. RDEVENTTM (Unlatched, DECIMAL)\n");
    printf ("\t6. RDEVENTRTMEQ (Latched, DECIMAL)\n");
    printf ("\t7. RDEVENTTV (Unlatched, BINARY)\n");
    printf ("\t8. RDEVENTTVREQ (Latched, BINARY)\n\n");
    printf ("\t9. TIMEREQUEST (Latch Time)\n");
    printf ("\t10. EVENTREQUEST (Latch Event)\n");
    printf ("\n Enter Choice : ");
    scanf ("%i", &choice);

    switch (choice)
    {
        case 1:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDTIMETM;
            arg_choice = (int) &tfpTM;
            break;
        case 2:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDTIMETMREQ;
            arg_choice = (int) &tfpTM;
            break;
        case 3:
            print_choice = TIME_BINARY;
            ioctl_choice = RDTIMETV;
            arg_choice = (int) &tfpTV;
            break;
        case 4:
            print_choice = TIME_BINARY;
            ioctl_choice = RDTIMETVREQ;
            arg_choice = (int) &tfpTV;
            break;
        case 5:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDEVENTTM;
            arg_choice = (int) &tfpTM;
            break;
        case 6:
            print_choice = TIME_DECIMAL;
            ioctl_choice = RDEVENTTMREQ;
            arg_choice = (int) &tfpTM;
            break;
        case 7:
            print_choice = TIME_BINARY;
            ioctl_choice = RDEVENTTV;
            arg_choice = (int) &tfpTV;
            break;
        case 8:
            print_choice = TIME_BINARY;
            ioctl_choice = RDEVENTTVREQ;
            arg_choice = (int) &tfpTV;
            break;
        case 9:
            print_choice = TIME_BINARY + TIME_DECIMAL;  /* none */
            ioctl_choice = TIMEREQUEST;
            arg_choice = (int) 0;
            break;
        case 10:
            print_choice = TIME_BINARY + TIME_DECIMAL;  /* none */
            ioctl_choice = EVENTREQUEST;
            arg_choice = (int) 0;
            break;

        default:
            printf ("Not implemented\n");
            return;
    }

    if (ioctl (fd, ioctl_choice, arg_choice) == ERROR)
    {
        (void) printf ("ioctl (%d) failed: ", ioctl_choice);
        printErrno (errnoGet ());
        return;
    }

    if (print_choice == TIME_DECIMAL)
        (void) tmPrint (&tfpTM);
    else if (print_choice == TIME_BINARY)
        (void) tvPrint (&tfpTV);

}

/**************************************************************************************/
/* Description : pci_strobe_routines */
/**************************************************************************************/
void pci_strobe_routines ()
{
int strobe_secs;
TFP_TM tfpTM;

printf("\n\tEnter number of seconds till strobe : ");
scanf("%i", &strobe_secs);

if ((strobe_secs < 1) || (strobe_secs > 59))
   /* can't do this */
{
   printf("strobe must be between 1 and 59 seconds for this test \n");
   return;
}

ioctl (fd, SELTIMEFORMAT, TIME_DECIMAL); /* decimal time format */

/* Get current time */
ioctl (fd, RDTIMETMREQ, (int)&tfpTM);
printf("\nCurrent time is: \n");
tmPrint(&tfpTM);

if (tfpTM.tm.tm_sec + strobe_secs < 60)
   tfpTM.tm.tm_sec = tfpTM.tm.tm_sec + strobe_secs;
else
{
   tfpTM.tm.tm_sec = tfpTM.tm.tm_sec + strobe_secs - 60;
   if (tfpTM.tm.tm_min < 59)
      tfpTM.tm.tm_min = tfpTM.tm.tm_min + 1; /* doesn't handle the roll over */
   else
   {
      printf("Test program is not smart enough to strobe now\n");
      return;
   }
}
   
printf ("\nStrobe time is: \n");
tmPrint(&tfpTM);

ioctl (fd, SETSTROBETM, (int)&tfpTM); 

ioctl (fd, BC635_INT_WAITONSTROBE, 0); /* Should be a task that starts up before the
                                strobe - but since our example is in seconds
                                we have time to it after */

printf("Got Strobe\n");
}

/**************************************************************************************/ 
/* Description : pci_interrupt_routines */ 
/**************************************************************************************/ 
void pci_interrupt_routines ()
{
    int choice;

    printf ("\n\n");
    printf ("1. Wait on Event\n");
    printf ("2. Wait on Periodic\n"); 
    printf ("3. Wait on Strobe\n"); 
    printf ("4. Wait on 1 PPS\n"); 
    printf ("5. Wait on GPS\n"); 
    printf ("   Select : ");
    scanf ("%i", &choice);

    switch (choice)
    {
    case 1:
        ioctl (fd, BC635_INT_WAITONEVENT, 0);
        printf ("Got EVENT\n");
        break;
    case 2:
      {
        int i;
        for (i=0; i < 5; i++)
        {
           ioctl (fd, BC635_INT_WAITONPERIODIC,0);
           printf ("Got PERIODIC\n");
        }
      }
        break;
    case 3:
        ioctl (fd, BC635_INT_WAITONSTROBE,0);
        printf ("Got STROBE\n");
        break;
    case 4:
      {    
        int i;
        for (i=0; i < 5; i++)
        {
           ioctl (fd, BC635_INT_WAITON1PPS, 0);
           printf ("Got 1PPS\n");
        }
      }
        break;
    case 5:
        ioctl (fd, BC635_INT_WAITONGPS, 0);
        printf ("Got GPS\n");
        break;
    default:
        break;
    }
 
}
/**************************************************************************************/
/* Description : pci_intstat_routines() */
/**************************************************************************************/
void pci_intstat_routines ()
{
     int choice;
     UINT32 intstat;
 
     printf ("\n\n");
     printf ("1. Read Intstat \n");
     printf ("2. Set Intstat \n");
     printf ("3. Read Intmask \n");
     printf ("4. Set Intmask \n");
     printf ("   Select : ");
     scanf ("%i", &choice);
 
     switch (choice)
     {
       case 1:
          intstat = 0;
          ioctl (fd, RDINTSTAT, (int)&intstat);
          printf ("INTSTAT = %x\n", intstat & INT_BITMASK); 
          break;
       case 2:
          printf ("/nEnter value : ");
          scanf ("%i", &intstat);
          ioctl (fd, SETINTSTAT, intstat);
          break;
       case 3:   
          intstat = 0;   
          ioctl (fd, RDINTMASK, (int)&intstat);
          printf ("INTMASK = %x\n", intstat & INT_BITMASK);
          break;
       case 4: 
          printf ("/nEnter value : "); 
          scanf ("%i", &intstat);
          ioctl (fd, SETINTMASK, intstat);
          break;

       default:
          break;
      }
}
/**************************************************************************************/
/* Description : Main Menu */
/**************************************************************************************/
void pci_sw_reset ()
{
   ioctl (fd, SOFTWARERESET, 0);
}

void pci_set_offsets ()
{
   UINT32 offset;
   UINT32 halfhour;
   int    choice;
   printf ("\n\n 1. Set Generator time offset \n");
   printf ("2. Set Local Time Offset \n");
   printf ("   Select : ");
   scanf ("%i", &choice);

   printf ("Enter Offset : ");
   scanf ("%i", &offset);
   printf ("Enter Half Hour (0 or 1) : ");
   scanf ("%i", &halfhour);

   switch (choice) 
   {
      case 1:
       {
         struct tcgenoffset genoffset;

         genoffset.offset = (INT16) offset;
         genoffset.half_hour = (UINT8) halfhour;

         ioctl (fd, SETGENTMOFFSET, (int)&genoffset);
       
       }
         break;
      case 2:
      {
        struct loctmoffset locoffset;

        locoffset.offset = (INT16) offset;
        locoffset.half_hour = (UINT8) halfhour;

        ioctl (fd, SETLOCTMOFFSET, (int) &locoffset);

      }
         break;
      default:
         break;
   }
}

void pci_set_prop_delay()
{
  UINT32 delay;

  printf("Enter delay : ");
  scanf ("%i", &delay);

  ioctl (fd, SETPROPDELAY, delay);
}

void pci_set_leapsec_event ()
{
   INT32 tmp;
   UINT32 utmp;
   struct leapseconds leapsec;

   printf ("Enter Leap Sec Flag : ");
   scanf ("%i", &tmp);
   leapsec.ls_flag = (INT8) tmp;
   printf ("Enter Leap Time (Unix Format) : ");
   scanf ("%i", &utmp);
   leapsec.leap_tm = tmp;

   ioctl (fd, SETLEAPSECEVENT, (int) &leapsec);
   
   
}

void pci_gps_example ()
{
    TFP_GPS_PACKET gps_packet;
    UINT32 id;
    int i;

    printf("Enter Packet to receive (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.packet_id = id;
    gps_packet.packet_length = 0;
    
    ioctl (fd, REQGPSPACKET, (int)&gps_packet);

    if (gps_packet.packet_length == 0)
       printf("GPS Example - Could not receive GPS System Message\n"); 

    else
    {
       printf("Packet id = %x, lenght=%d\n", gps_packet.packet_id, gps_packet.packet_length);
       for (i=0; i < gps_packet.packet_length-1; i++)
           printf ("[%x] = %x\n", i, gps_packet.packet_data[i]);
    }
}

void pci_gps_example2 ()
{
    TFP_GPS_PACKET gps_packet;
    UINT32 id;  
    int i,tmp;
 
    printf("Enter Packet to Send (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.packet_id = id;
    printf("Enter Packet Length (decimal plus one) : ");
    scanf ("%d", &tmp);
    gps_packet.packet_length = (UINT8) tmp;
    for (i=0; i<gps_packet.packet_length-1;i++)
    {
       printf("Enter data (in hex) [%d] : ", i);
       scanf ("%x", &tmp);
       gps_packet.packet_data[i] = (UINT8) tmp;
    }
   
    ioctl (fd, SENDGPSPACKET, (int)&gps_packet);
 
}

void pci_gps_example3 () 
{
    TFP_GPS_SENDRECEIVE_PACKET gps_packet;
    UINT32 id;
    int i,tmp;

    printf("Enter Packet to Send (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.send.packet_id = id;
    printf("Enter Packet Length (decimal plus one) : ");
    scanf ("%d", &tmp);
    gps_packet.send.packet_length = (UINT8) tmp;
    for (i=0; i<gps_packet.send.packet_length-1;i++)
    {
       printf("Enter data (in hex) [%d] : ", i);
       scanf ("%x", &tmp);
       gps_packet.send.packet_data[i] = (UINT8) tmp;
    }
    
    printf("Enter Packet to receive (hex - i.e. 4a) : ");
    scanf ("%x", &id);
    gps_packet.receive.packet_id = id;
    gps_packet.receive.packet_length = 0;


    
    ioctl (fd, MANREQGPSPACKET, (int)&gps_packet);

    if (gps_packet.receive.packet_length == 0)
       printf("GPS Example - Could not receive GPS System Message\n");
 
    else
    {   
       printf("Packet id = %x, lenght=%d\n", gps_packet.receive.packet_id, gps_packet.receive.packet_length);
       for (i=0; i < gps_packet.receive.packet_length-1; i++)
           printf ("[%x] = %x\n", i, gps_packet.receive.packet_data[i]);
    }

    

}

void pci_set_gps_mode ()
{

  int mode;

  printf ("\n\t 1) Disable");
  printf ("\n\t 2) Enable");
  printf ("\n\t    Select : ");
  scanf ("%i", &mode);

  ioctl (fd, SETGPSMDFLG, mode);
}


void pci_set_gps_time_format()
{
   
}

void pci_set_clock_source()
{
int clock;

printf ("\n\t Enter Clock Source (Internal = 1, External=2) : ");
scanf ("%i", &clock);

if (clock == 1)
   ioctl(fd, CONTROLTIMEBASE, CLOCK_INTERNAL);
else if (clock == 2)
   ioctl(fd, CONTROLTIMEBASE, CLOCK_EXTERNAL);
else
   printf ("Invalid value\n");
  
}

void pci_looping_time()
{
TFP_TV tfpTV;  
int i;

ioctl (fd, SELTIMEFORMAT, TIME_BINARY); /* binary time format */
 
for (i = 0; i <= 10; i ++)
{
   /* Get current time */
   ioctl (fd, RDTIMETVREQ, (int)&tfpTV);
   printf("\nCurrent time is: \n");
   tvPrint(&tfpTV);
   taskDelay(60*2); /* every 2 seconds */
}
}

/**************************************************************************************/
/* Description : Main Menu */
/**************************************************************************************/
int 
menu_main ()
{
    int     menu;

    do
    {
        printf ("\n  Datum Inc. Bancomm-Timing Division\n\n");
        printf ("    bc635PCI Time & Frequency Processor\n\n");
        printf ("1.  Read time/Event Functions    14. Set Offsets \n");
        printf ("2.  Select Operational Mode      15. Prop Delay \n");
        printf ("3.  Select Time Code Format      16. Leap Sec \n");
        printf ("4.  Select Output Freq           17. Capunlock \n");
        printf ("5.  Select Time Format           18. GPS Example1 \n");
        printf ("6.  Program Heartbeat            19. Set GPS Mode \n");
        printf ("7.  Set Time Functions           20. Set GPS TM Fmt\n");
        printf ("8.  Select Clock Source          21. GPS Example2\n");
        printf ("9.  Current Settings             22. GPS Example3\n");
        printf ("10. Strobe                       23. Looping Time\n");
        printf ("11. Interrupt Routines\n");
        printf ("12. Interrupt Stat Routines\n");
        printf ("13. Software reset\n");
        printf ("0.  Exit the Program \n\n");
        printf ("    Select: ");
        scanf ("%i", &menu);

        switch (menu)
        {
            case 1:
                read_time_menu ();
                break;
            case 2:
                pci_mode ();
                break;
            case 3:
                pci_time_code ();
                break;
            case 4:
                pci_out_freq ();
                break;
            case 5:
                pci_time_format ();
                break;
            case 6:
                pci_heartbeat ();
                break;
            case 7:
                pci_set_time ();
                break;
            case 8:             
                pci_set_clock_source(); 
                break;
            case 9:
                pci_req_current_settings ();
                break;
            case 10:
                pci_strobe_routines ();
                break;
            case 11:
                pci_interrupt_routines ();
                break;
            case 12:
                pci_intstat_routines ();
                break;
            case 13:
                pci_sw_reset ();
                break;
            case 14:
                pci_set_offsets ();
                break;
            case 15:
                pci_set_prop_delay();
                break;
            case 16: 
                pci_set_leapsec_event();
                break;
            case 17:
                ioctl (fd, CAPUNLOCK, 0);
                break;
            case 18:
                pci_gps_example();
                break;
            case 19:
                pci_set_gps_mode();
                break;
            case 20:
                pci_set_gps_time_format();
                break;
             case 21:
                pci_gps_example2();
                break;
            case 22:
                pci_gps_example3();
                break;
            case 23:
                pci_looping_time();
                break;
            case 0:
            default:
                return OK;
        }
    } while (1);
}


